#version 330 core

//layout(std140) uniform GlobalLights
//{
//	uniform vec3 ambientI;
//};

uniform sampler2D colorTexture;
uniform vec3 lightPos; // light position in world space
uniform vec3 lightI;
uniform vec3 camPos;

in vec3 theNormal;
in vec2 fragmentUV;
in vec3 fragmentPos; // position of fragment in world space

layout (location = 0) out vec4 outputColor;
layout (location = 1) out vec4 outputMotion;

const float ambient = 0.05;
const float lightExp = 800; // light expansion
const float lAtt = 1.0 / 300; // linear attenuation coefficient
const float qAtt = 1.0 / 800; // quadratic attenuation coefficient
const float k_d = 0.8; // diffuse coefficient
const float p = 8; // specular parameter
const float k_s = 0.7; // specular reflection coefficient

const float texSize = 5; // the texture is stretched over texSize units
const float texSizeMul = 1 / (2 * texSize);

void main()
{
	vec3 c = texture(colorTexture, vec2((fragmentPos.x + fragmentPos.y) * texSizeMul, (fragmentPos.z + fragmentPos.y) * texSizeMul)).rgb;
	vec3 l = normalize(lightPos - fragmentPos);
	float nl = clamp(dot(normalize(theNormal), l), 0, 1);

	// point light
	float lightIb = (lightI.x + lightI.y + lightI.z) / 3;
	float lightDist = length(lightPos - fragmentPos);
	float lightIAtt = lightIb - min(lAtt * lightDist + min(qAtt * pow(lightDist, 2), 0.8), lightIb);

	vec3 v = normalize(camPos - fragmentPos);
	vec3 h = normalize(l + v);

	outputColor = vec4(ambient * c + k_d * nl * c * lightIAtt + k_s * lightIAtt * pow(clamp(dot(theNormal, h), 0, 1), p), 1);
	outputMotion = vec4(0);
}
